/*
*  memory_controller.h - Generic memory controller driver for sysfs
* ($Revision: 1 $)
*
*  Copyright (C) 2006, 2007 Sujith Thomas <sujith.thomas@intel.com>
*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; version 2 of the License.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*  General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, write to the Free Software Foundation, Inc.,
*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Sysfs class driver for memory_controller. Exposes attributes for get/set
* memory bandwidth
*
*
*/

#include <linux/device.h>

struct memory_controller_device;

/* Callback functions to the actual memory device driver */
struct memory_controller_ops {
	int (*get_max_bandwidth) (struct memory_controller_device *device,
				  char *buf);
	int (*get_cur_bandwidth) (struct memory_controller_device *device,
				  char *buf);
	int (*set_cur_bandwidth) (struct memory_controller_device *device,
				  const char *buf);
};

struct memory_controller_device {
	struct memory_controller_ops *ops;	/* Callback routines */
	struct class sysfs_class;	/* '/sys/class' entry*/
	struct class_device class_dev;	/*class device entry in sys/class*/
};

/*
 * memory_controller_device_register
 * --------------------------------
 * Method for registering memory_controller class of devices with sysfs
 * name: The name that should appear in sysfs
 * dev : device* on which sysfs folder should appear
 * devdata : Device private context
 * ops : List of call back functions for various attributes
 * Returns either an
 * ERR_PTR() or a pointer to the newly allocated device.
 */
struct memory_controller_device *
memory_controller_device_register(const char *name
				  , struct device *dev
				  , void *devdata
				  , struct memory_controller_ops *ops);

/*
 * memory_controller_device_unregister
 * ----------------------------------
 * Method for unregistering memory_controller devices  with sysfs
 * mc: Pointer to memory_controller device
 */
void memory_controller_device_unregister(struct memory_controller_device
					 *device);

#define to_memory_controller_device(obj) \
container_of(obj, struct memory_controller_device, class_dev)
